#ifndef _LIDAR_LITE_H
#define _LIDAR_LITE_H

#include <iostream>
#include <sstream>  
#include <string>    
#include <iomanip>
#include <usb_i2c.h>
#include "exceptions/lidar_lite_exceptions.h"

// The device has a 7-bit slave address with a default value of 0x62.
unsigned char DEVICE_ID = 0x62;

enum V3_REGISTER_DEF
{  
	ACQ_COMMAND = 		0x00,	// Device command.
	STATUS = 			0x01,	// System status.
	SIG_COUNT_VAL = 	0x02,	// Maximum acquisition count.
	ACQ_CONFIG_REG = 	0x04,	// Acquisition mode control.
	VELOCITY = 			0x09,	// Velocity measurement output.
	PEAK_CORR = 		0x0c,	// Peak value in correlation record.
	NOISE_PEAK = 		0x0d,	// Correlation record noise floor.
	SIGNAL_STRENGTH = 	0x0e,	// Received signal strength.
	FULL_DELAY_HIGH = 	0x0f,	// Distance measurement high byte.
	FULL_DELAY_LOW = 	0x10,	// Distance measurement low byte.
	OUTER_LOOP_COUNT = 	0x11,	// Burst measurement count control.
	REF_COUNT_VAL = 	0x12,	// Reference acquisition count.
	LAST_DELAY_HIGH = 	0x14,	// Previous distance measurement high byte.
	LAST_DELAY_LOW = 	0x15,	// Previous distance measurement low byte.
	UNIT_ID_HIGH = 		0x16,	// Serial number high byte.
	UNIT_ID_LOW = 		0x17,	// Serial number low byte.
	I2C_ID_HIGH = 		0x18,	// Write serial number high byte for I2C address unlock.
	I2C_ID_LOW = 		0x19,	// Write serial number low byte for I2C address unlock.
	I2C_SEC_ADDR = 		0x1a,	// Write new I2C address after unlock.
	THRESHOLD_BYPASS = 	0x1c,	// Peak detection threshold bypass.
	I2C_CONFIG = 		0x1e,	// Default address response control.
	COMMAND = 			0x40,	// State command.
	MEASURE_DELAY = 	0x45,	// Delay between automatic measurements.
	PEAK_BCK = 			0x4c,	// Second largest peak value in correlation record.
	CORR_DATA = 		0x52,	// Correlation record data low byte.
	CORR_DATA_SIGN = 	0x53,	// Correlation record data high byte.
	ACQ_SETTINGS =		0x5d,	// Correlation record memory bank select.
	POWER_CONTROL =		0x65, 	// Power state control.
};

enum DEV_STATUS 
{  
	IDLE,
	RUNNING
};

class CLidarLite
{
  private:
	std::string serial_;	// Serial port (e.g., run dmesg).
	CUSBI2C* adapter_;		// Device object.
	int status_;			// Device status.

	// Blinks the usb_i2c adapter led.	
	void blink_led_usb_adapter(void);

	// Converts an HEX number (int object) to string. Print purposes.
	std::string hex_to_str(const int &val);

	// Writes a command (cmd) to the register of the I2C device (addr).
	void write(unsigned char addr, unsigned char cmd);

	// Read byte/s (len=[1,2]) from the register of the I2C device (addr).
	int read(unsigned char addr, int len);

    // Successive 2-byte readings (Autoincrement of address: A note about 0x8f vs 0x0f)
    // Set the highest bit of any register to "1" if you set the high byte of a register 
    // and then take succesive readings from that register, then LIDAR-Lite automatically 
    // increments the register one for each read. 
    // An example: If we want to read the high and low bytes for the distance, we could 
    // take two single readings from 0x0f and 0x10, or we could take 2 byte read from 
    // register 0x8f. 0x8f = 10001111 and 0x0f = 00001111, meaning that 0x8f is 0x0f with 
    // the high byte set to "1", ergo it autoincrements.
	unsigned short int addr_to_read_2bytes(unsigned char first_byte_addr);

  public:
    CLidarLite(const std::string &serial);
    ~CLidarLite(void);

	void open(void);
	void close(void);
	void reset(void);

	// The maximum acquisition count limits the number of times the device will
	// integrate acquisitions to find a correlation record peak (from a returned signal),
	// which occurs at long range or with low target reflectivity. This controls the
	// minimum measurement rate and maximum range. The unit-less relationship
	// is roughly as follows: rate = 1/n and range = n^(1/4), where n is the number of
	// acquisitions.
	void set_max_acq_count(const unsigned char &num_acq);

};

#endif
